#!/bin/bash

# COPYRIGHT 2003-2012, EMULEX CORPORATION                               
# 3333 Susan St., Costa Mesa, CA 92626                                  
#                                                                         
# All rights reserved.  This computer program and related documentation 
# is protected by copyright  and distributed under licenses restricting 
# its use,  copying,  distribution  and decompilation.    This computer 
# program  and its  documentation  are CONFIDENTIAL  and a TRADE SECRET 
# of EMULEX CORPORATION.   The receipt or  possession of  this  program 
# or its documentation does not  convey rights to reproduce or disclose 
# its  contents,  or to  manufacture, use, or sell anything that it may 
# describe, in whole or in part,  without the specific  written consent 
# of  EMULEX CORPORATION.   Any reproduction  of  this program  without 
# the express  written  consent  of EMULEX  CORPORATION  is a violation 
# of the  copyright laws  and may  subject you to  criminal prosecution.

#######################################################################
#                           linddcfg.sh                               #
# This script executes linddcfg with the requested arguments. This    #
# script also installs the MILI daemon and preserves the daemons's    #
# initial state.                                                      #
#                                                                     #
#######################################################################

# Status vars
LPCFG_STATUS=0
SUCCESS=0
TRUE=1
FALSE=0
FIELD_ADMIN_RIGHTS_ERROR=15
FIELD_UNSUPPORTED_OS=23
FIELD_UNSUPPORTED_ARCH=24
FIELD_BIN_NOT_FOUND=25
BYPASS_MILI_INSTALL=0

# kit directory vars
CLU_OS=""
CLU_OS_FULL=""
CLU_ARCH=""
LPFC_OS=""
KIT_DIR=""


# lpcfg vars
LOAD_PATH="LD_LIBRARY_PATH=."
LPCFG_BIN="linddcfg"
ARGUMENTS=""

# driver vars
LPFC_DRIVER=""

# logfile
LOG_FILE="field_lpcfg.log"
LOG=""

# OCM daemons
ELX_DISCO_SCRIPT=/etc/init.d/elxdiscoveryd
ELX_HBAMGR_SCRIPT=/etc/init.d/elxhbamgrd
ELX_SNMP_SCRIPT=/etc/init.d/elxsnmpd
ELX_MILI_SCRIPT=/etc/init.d/elxmilid
INITIAL_MILI_STATUS=0
INITIAL_SNMP_STATUS=0
INITIAL_HBAMGR_STATUS=0
INITIAL_DISCO_STATUS=0




#####################################################################
#                                                                   #
# Function: is_root                                                 #
#                                                                   #
# Checks if this script is being run by root.                       #
#                                                                   #
#####################################################################
function is_root()
{
    USER=`whoami`
    if [ "$USER" != "root" ]; then
        echo "Not the root user! Exiting..."
        exit $FIELD_ADMIN_RIGHTS_ERROR
    fi
}


#####################################################################
#                                                                   #
# Function: log_message                                             #
#                                                                   #
# Print the strings passed as parameters to a log file.             #
#                                                                   #
#####################################################################
function log_message()
{
    local ECHO_OPTIONS
    
    if [ $# -eq 0 ] ; then
        echo "" >> ${LOG}
    elif [ -z "$1" ] ; then
        echo "" >> ${LOG}
    elif [ "$1" = "-init" ] ; then
        rm -f $LOG
        print_time "Start_time"
    else
        while [ -n "$1" ] ; do
            if [ "${1:0:1}" = "-" ] ; then
                ECHO_OPTIONS="${1}"
            else
                echo ${ECHO_OPTIONS} "$1" >> ${LOG}
            fi
            shift
        done
    fi
    return $SUCCESS
}


#####################################################################
#                                                                   #
# Function: print_time                                              #
#                                                                   #
# Prints the current time.                                          #
#                                                                   #
#                                                                   #
#####################################################################
function print_time()
{
    echo "[$1]" >> ${LOG}
    echo -e "$(date)\n" >> ${LOG}
}


#####################################################################
#                                                                   #
# Function: print_cli                                               #
#                                                                   #
# Prints the command line input.                                    #
#                                                                   #
#                                                                   #
#####################################################################
function print_cli()
{
    log_message "[print_cli]"
    log_message -e "$0 $ARGUMENTS\n"
}


#####################################################################
#                                                                   #
# Function: set_kit_path                                            #
#                                                                   #
# Sets the kit's directory path.                                    #
#                                                                   #
#####################################################################
function set_kit_path()
{
    KIT_DIR=$(cd `dirname $0` && pwd)
}


#####################################################################
#                                                                   #
# Function: init_log_file                                           #
#                                                                   #
# Initializes the log file.                                         #
#                                                                   #
#####################################################################
function init_log_file
{
    LOG=$KIT_DIR/$LOG_FILE
    log_message -init
}


#####################################################################
#                                                                   #
# Function: set_dir_paths                                           #
#                                                                   #
# Sets the common and config diretory paths.                        #
#                                                                   #
#####################################################################
function set_dir_paths()
{
    log_message "[set_dir_paths]"
    LPCFG_PAYLOAD="$KIT_DIR/../common/payload"
    LPCFG_CONFIG="$KIT_DIR"
    
    log_message "cp -r --preserve=all --remove-destination $LPCFG_PAYLOAD/* $KIT_DIR > /dev/null 2>&1"
    cp -r --preserve=all --remove-destination $LPCFG_PAYLOAD/* $KIT_DIR > /dev/null 2>&1
    
    LPCFG_PAYLOAD="$KIT_DIR"
    
    log_message "Payload dir = $LPCFG_PAYLOAD"
    log_message "Config dir  = $LPCFG_CONFIG"
    log_message "Kit dir     = $KIT_DIR"
    log_message ""
}


#####################################################################
#                                                                   #
# Function: log_script_dir_contents                                 #
#                                                                   #
# Log the contents of the execution script's directory.             #
#                                                                   #
#####################################################################
function log_script_dir_contents()
{
    log_message "[log_script_dir_contents]"
    log_message "ls -Rl \"$KIT_DIR\" >> \"$LOG\" 2>&1"
    ls -Rl "$KIT_DIR" >> "$LOG" 2>&1
    log_message ""
}


#####################################################################
#                                                                   #
# Function: determine_arch                                          #
#                                                                   #
# Determine the processor architecture.                             #
#                                                                   #
# NOTE: If an invalid architecture is detected this function will   #
#       terminate script execution.                                 #
#                                                                   #
#####################################################################
function determine_arch()
{
    log_message "[determine_arch]"
    CLU_ARCH=$(uname -m)
    case ${CLU_ARCH} in
        i[3456]86)
            CLU_ARCH=i386
            ARCH_STR=
            ;;
        
        x86_64)
            CLU_ARCH=x86_64
            ARCH_STR=_64
            ;;
        
        *)
            log_message "Unsupported architecture: $CLU_ARCH"
            log_message "Aborting execution."
            exit $FIELD_UNSUPPORTED_ARCH
            ;;
    esac
    
    log_message -e "CLU_ARCH = $CLU_ARCH\n"
}


#####################################################################
#                                                                   #
# Function: determine_distribution                                  #
#                                                                   #
# Determine the distribution.                                       #
#                                                                   #
# NOTE: If an invalid distribution is detected this function will   #
#       terminate script execution.                                 #
#                                                                   #
#####################################################################
function determine_distribution()
{
    log_message "[determine_distribution]"
    
    if [ -f /etc/redhat-release ]; then
        maj_distro=$(uname -r | awk -F- '{print$1}' | awk -F. '{print$3}')
        min_distro=$(uname -r | awk -F- '{print$2}' | awk -F. '{print$1}')
        if [ $maj_distro -eq 18 ]; then
            # Kernel 2.6.18
            CLU_OS="rhel-5"
            
            if [[ $min_distro -ge 194 && $min_distro -lt 238 ]]; then
                CLU_OS_FULL="rhel-5.5"
            elif [[ $min_distro -ge 238 && $min_distro -lt 274 ]]; then
                CLU_OS_FULL="rhel-5.6"
            elif [ $min_distro -ge 274 ]; then
                CLU_OS_FULL="rhel-5.7 or later"
            else
                print_unsupported_kernel "RHEL"
            fi
            
        elif [ $maj_distro -eq 32 ]; then
            # Kernel 2.6.32
            CLU_OS="rhel-6"
            
            if [[ $min_distro -ge 71 && $min_distro -lt 131 ]]; then
                CLU_OS_FULL="rhel-6.0"
            elif [[ $min_distro -ge 131 && $min_distro -lt 220 ]]; then
                CLU_OS_FULL="rhel-6.1"
            elif [ $min_distro -ge 220 ]; then
                CLU_OS_FULL="rhel-6.2 or later"
            else
                print_unsupported_kernel "RHEL"
            fi
        else
            print_unsupported_kernel "RHEL"
        fi
        
        
    elif [ -f /etc/SuSE-release ]; then
        kernel_ver=$(uname -r | awk -F- '{print$1}' | awk -F. '{print$1}')
        maj_distro=$(uname -r | awk -F- '{print$1}' | awk -F. '{print$3}')
        min_distro=$(uname -r | awk -F- '{print$1}' | awk -F. '{print$4}')
        sub_distro=$(uname -r | awk -F- '{print$2}' | awk -F. '{print$2}')
        
        if [ $kernel_ver -eq 3 ]; then
            # Kernel 3.0.13
            CLU_OS="sles-11"
            CLU_OS_FULL="sles-11.2 or later"
        
        elif [ $maj_distro -eq 32 ]; then
            # Kernel 2.6.32
            CLU_OS="sles-11"
            
            if [ $min_distro -ge 12 ]; then
                CLU_OS_FULL="sles-11.1 or later"
            else
                print_unsupported_kernel "SLES"
            fi
        
        elif [ $maj_distro -eq 16 ]; then
            # Kernel 2.6.16
            CLU_OS="sles-10"
            
            if [ $min_distro -eq 60 ]; then
                if [ $sub_distro -ge 54 ] && [ $sub_distro -lt 85 ]; then
                    CLU_OS_FULL="sles-10.3"
                elif [ $sub_distro -ge 85 ]; then
                    CLU_OS_FULL="sles-10.4 or later"
                else
                    print_unsupported_kernel "SLES"
                fi
            else
                print_unsupported_kernel "SLES"
            fi
        else
            print_unsupported_kernel "SLES"
        fi
        
    fi
    
    if [ ! -d ${KIT_DIR}/${CLU_ARCH}/${CLU_OS} ]; then
        log_message "Could not find installation directory: ${KIT_DIR}/${CLU_ARCH}/${CLU_OS}"
        log_message "Aborting execution."
        exit $FIELD_BIN_NOT_FOUND
    fi
    
    log_message "APPS DIR        = ${KIT_DIR}/${CLU_ARCH}/${CLU_OS}"
    log_message "CLU_OS_FULL     = $CLU_OS_FULL"
    log_message ""
}


#####################################################################
#                                                                   #
# Function: print_unsupported_kernel                                #
#                                                                   #
# Print an error messsage and exit.                                 #
#                                                                   #
# NOTE: This function will terminate script execution.              #
#                                                                   #
#####################################################################
function print_unsupported_kernel()
{
    distro=$1
    kernel=$(uname -r)
    log_message "Unsupported $distro Kernel = $kernel"
    log_message "Aborting execution."
    exit $FIELD_UNSUPPORTED_OS
}


#####################################################################
#                                                                   #
# Function: get_daemons_status                                      #
#                                                                   #
# Determines the initial status of the OCM daemons.                 #
#                                                                   #
#####################################################################
function get_daemons_status()
{
    log_message "[get_daemons_status]"
    
    if [ $BYPASS_MILI_INSTALL -eq $TRUE ]; then
        log_message -e "Bypassing stopping the daemons!\n"
        return 0
    fi

    ps -ef | grep elxdiscoveryd | grep -v grep > /dev/null
    if [ $? -eq $SUCCESS ]; then
        INITIAL_DISCO_STATUS=$TRUE
        log_message "Stopping elxdiscoveryd"
        log_message "$ELX_DISCO_SCRIPT stop >> ${LOG} 2>&1"
        $ELX_DISCO_SCRIPT stop >> ${LOG} 2>&1
    fi
    
    ps -ef | grep elxhbamgrd | grep -v grep > /dev/null
    if [ $? -eq $SUCCESS ]; then
        INITIAL_HBAMGR_STATUS=$TRUE
        log_message "Stopping elxhbamgrd"
        log_message "$ELX_HBAMGR_SCRIPT stop >> ${LOG} 2>&1"
        $ELX_HBAMGR_SCRIPT stop >> ${LOG} 2>&1
    fi
    
    ps -ef | grep elxsnmpd | grep -v grep > /dev/null
    if [ $? -eq $SUCCESS ]; then
        INITIAL_SNMP_STATUS=$TRUE
        log_message "Stopping elxsnmpd"
        log_message "$ELX_SNMP_SCRIPT stop >> ${LOG} 2>&1"
        $ELX_SNMP_SCRIPT stop >> ${LOG} 2>&1
    fi
    
    ps -ef | grep mili2d | grep -v grep > /dev/null
    if [ $? -eq $SUCCESS ]; then
        INITIAL_MILI_STATUS=$TRUE
        log_message "Stopping elxmilid"
        log_message "$ELX_MILI_SCRIPT stop >> ${LOG} 2>&1"
        $ELX_MILI_SCRIPT stop >> ${LOG} 2>&1
    fi
    
    log_message "INITIAL_DISCO_STATUS  = $INITIAL_DISCO_STATUS"
    log_message "INITIAL_HBAMGR_STATUS = $INITIAL_HBAMGR_STATUS"
    log_message "INITIAL_SNMP_STATUS   = $INITIAL_SNMP_STATUS"
    log_message "INITIAL_MILI_STATUS   = $INITIAL_MILI_STATUS"
    log_message ""
}


#####################################################################
#                                                                   #
# Function: restore_daemons_status                                  #
#                                                                   #
# Restores the initial status of the OCM daemons.                   #
#                                                                   #
#####################################################################
function restore_daemons_status()
{
    log_message "[restore_daemons_status]"
    
    if [ $INITIAL_MILI_STATUS -eq $TRUE ]; then
        log_message "$ELX_MILI_SCRIPT start >> ${LOG} 2>&1"
        $ELX_MILI_SCRIPT start >> ${LOG} 2>&1
    fi
    if [ $INITIAL_SNMP_STATUS -eq $TRUE ]; then
        log_message "$ELX_SNMP_SCRIPT start >> ${LOG} 2>&1"
        $ELX_SNMP_SCRIPT start >> ${LOG} 2>&1
    fi
    if [ $INITIAL_HBAMGR_STATUS -eq $TRUE ]; then
        log_message "$ELX_HBAMGR_SCRIPT start >> ${LOG} 2>&1"
        $ELX_HBAMGR_SCRIPT start >> ${LOG} 2>&1
    fi
    if [ $INITIAL_DISCO_STATUS -eq $TRUE ]; then
        log_message "$ELX_DISCO_SCRIPT start >> ${LOG} 2>&1"
        $ELX_DISCO_SCRIPT start >> ${LOG} 2>&1
    fi
    
    log_message ""
}


#####################################################################
#                                                                   #
# Function: install_milid                                           #
#                                                                   #
# Installs the MILI daemon.                                         #
#                                                                   #
# Returns:                                                          #
#         0 -- Successfully started the MILI daemon.                #
#         1 -- Error in starting the MILI daemon.                   #
#                                                                   #
#####################################################################
function install_milid()
{
    log_message "[install_milid]"
    current_dir=$(pwd)
       
    log_message "Starting MILI daemon..."
    log_message "cd ${KIT_DIR}/${CLU_ARCH}/${CLU_OS}"
    cd ${KIT_DIR}/${CLU_ARCH}/${CLU_OS}
    
    log_message "./elxmilid start >> ${LOG} 2>&1"
    ./elxmilid start >> ${LOG} 2>&1
    
    if [ $? -ne $SUCCESS ]; then
        log_message "Warning: elxmilid failed to start!"
    fi
    
    cd $current_dir
    log_message ""
}


#####################################################################
#                                                                   #
# Function: uninstall_milid                                         #
#                                                                   #
# Uninstalls the MILI daemon.                                       #
#                                                                   #
# Returns:                                                          #
#         0 -- Successfully stopped and removed the MILI daemon.    #
#         1 -- Error in removing the MILI daemon.                   #
#                                                                   #
#####################################################################
function uninstall_milid()
{
    log_message "[uninstall_milid]"
    current_dir=$(pwd)
    
    log_message "Stopping MILI daemon..."
    log_message "cd ${KIT_DIR}/${CLU_ARCH}/${CLU_OS}"
    cd ${KIT_DIR}/${CLU_ARCH}/${CLU_OS}
    
    log_message "./elxmilid stop >> ${LOG} 2>&1"
    ./elxmilid stop >> ${LOG} 2>&1
    
    if [ $? -ne $SUCCESS ]; then
        log_message "Warning: elxmilid failed to stop!"
    fi
    
    cd $current_dir
    log_message ""
}


#####################################################################
#                                                                   #
# Function: check_for_help                                          #
#                                                                   #
# Checks an argument for the help (-h), version (-v), or            #
# verifyFW (-e) flag.                                               #
#                                                                   #
#                                                                   #
#####################################################################
function check_for_help()
{
    ARG=$1
    if [[ $ARG == "-h" ]] || \
       [[ $ARG == "-H" ]] || \
       [[ $ARG == "-v" ]] || \
       [[ $ARG == "-V" ]] || \
       [[ $ARG == "-e" ]] || \
       [[ $ARG == "-E" ]]; then
        BYPASS_MILI_INSTALL=$TRUE
    fi
}


#####################################################################
#                                                                   #
# Function: execute_lpcfg                                           #
#                                                                   #
# Execute lpcfg.                                                    #
#                                                                   #
# Returns:                                                          #
#         The linddcfg return code.                                 #
#                                                                   #
#####################################################################
function execute_lpcfg()
{
    log_message "[execute_lpcfg]"
    log_message "Executing: D_LIBRARY_PATH=$KIT_DIR/$CLU_ARCH/$CLU_OS $KIT_DIR/$CLU_ARCH/$CLU_OS/$LPCFG_BIN -p $LPCFG_PAYLOAD -c $LPCFG_CONFIG $ARGUMENTS "
    LD_LIBRARY_PATH=$KIT_DIR/$CLU_ARCH/$CLU_OS $KIT_DIR/$CLU_ARCH/$CLU_OS/$LPCFG_BIN -p $LPCFG_PAYLOAD -c $LPCFG_CONFIG $ARGUMENTS
    LPCFG_STATUS=$?
    log_message ""
}


#####################################################################
#                                                                   #
# Begin script execution                                            #
#                                                                   #
# Save the command line arguments                                   #
#                                                                   #
#####################################################################
for i in $*
do
    #echo "i = $i"
    if [ -z "$ARGUMENTS" ]; then
        ARGUMENTS=$i
        check_for_help $i
    else
        ARGUMENTS="${ARGUMENTS} $i"
    fi
done
   

#
# Check for the root user.
#
is_root


#
# Set the kit path.
#
set_kit_path


#
# Initialize the log file.
#
init_log_file


#
# Write the command line input to the log file.
#
print_cli


#
# Set the payload and config directories.
#
set_dir_paths


#
# Log the contents of the execution script's directory.
#
log_script_dir_contents


#
# Determine if this architecture is supported
#
determine_arch


#
# Determine if this operating system is supported
#
determine_distribution


#
# Save the initial daemon status
#
get_daemons_status


#
# Check for the help and version args
#
if [ $BYPASS_MILI_INSTALL -eq $TRUE ]; then
    log_message "[Bypass_MILI_Install]"
    log_message -e "Skipping MILI installation!\n"
    execute_lpcfg
    print_time "Stop_time"
    exit $LPCFG_STATUS
fi


#
# Install and start MILID
#
install_milid


#
# Execute linddcfg command
#
execute_lpcfg


#
# Stop and remove MILID
#
uninstall_milid


#
# Restore the initial daemon status
#
restore_daemons_status


#
# Write the completion time
#
print_time "Stop_time"

exit $LPCFG_STATUS
